@@ -1,7 +1,7 @@
 #ABSTRACT: A tiny plugin system for perl
 package Plugin::Tiny;
 {
-  $Plugin::Tiny::VERSION = '0.002';
+  $Plugin::Tiny::VERSION = '0.003';
 }
 use strict;
 use warnings;
@@ -9,6 +9,8 @@ use Carp 'confess';
 use Class::Load 'load_class';
 use Moose;
 use namespace::autoclean;
+use Scalar::Util 'blessed';
+#use Data::Dumper;
 
 
 has '_registry' => (    #href with phases and plugin objects
@@ -19,6 +21,9 @@ has '_registry' => (    #href with phases and plugin objects
 );
 
 
+has 'debug'=>(is=>'ro', isa=>'Bool', default=> sub{0});
+
+
 has 'prefix' => (is => 'ro', isa => 'Str');
 
 
@@ -29,47 +34,94 @@ has 'role' => (is => 'ro', isa => 'Str');
 # METHODS
 #
 
+
+
 sub register {
     my $self   = shift;
     my %args   = @_;
     my $plugin = delete $args{plugin} or confess "Need plugin";
+
     if ($self->prefix) {
         $plugin = $self->prefix . $plugin;
     }
-    my $phase = $args{phase} ? delete $args{phase} : _lastPart($plugin);
+    my $phase =
+      $args{phase}
+      ? delete $args{phase}
+      : $self->default_phase($plugin);
 
     my $role = $self->role if $self->role;    #default role
-    $role = delete $args{role} if $args{role};
-
-    load_class($plugin) or confess "Can't load $plugin";
-    $self->{_registry}{$phase} = $plugin->new(%args);
+    $role = delete $args{role} if exists $args{role};
 
+    load_class($plugin) or confess "Can't load '$plugin'";
+ 
     if ($role && !$plugin->does($role)) {
-        confess qq(Plugin doesn't plugin into role '$role');
+        confess qq(Plugin '$plugin' doesn't do role '$role');
     }
+    $self->{_registry}{$phase} = $plugin->new(%args) || confess "Can't make $plugin";
+    print "register $plugin [$phase]\n" if $self->debug;
     return $self->{_registry}{$phase};
 }
 
 
 
 sub get_plugin {
-    return $_[0]->{_registry}{$_[1]};
+    my $self = shift;
+    my $phase = shift or return;
+    return if (!$self->{_registry}{$phase});
+    return $self->{_registry}{$phase};
 }
 
 
+
+
+sub default_phase {
+    my $self   = shift;
+    my $plugin = shift;    #a class name
+
+    if ($self->prefix) {
+        my $phase  = $plugin;
+        my $prefix = $self->prefix;
+        $phase =~ s/$prefix//;
+        $phase =~ s/:://g;
+        return $phase;
+    }
+    else {
+        my @parts = split('::', $plugin);
+        return $parts[-1];
+    }
+}
+
+
+sub get_class {
+    my $self = shift;
+    my $plugin = shift or return;
+    blessed($plugin);
+}
+
+
+
+sub get_phase {
+    my $self         = shift;
+    my $plugin       = shift or return;
+    blessed($plugin);
+    my $current_class = $self->get_class($plugin);
+    #print 'z:['.join(' ', keys %{$self->{_registry}})."]\n";
+    foreach my $phase (keys %{$self->{_registry}}) {
+        my $registered_class=blessed ($self->{_registry}{$phase});
+        print "[$phase] $registered_class === $current_class\n";
+        return $phase if ("$registered_class" eq "$current_class");
+    }
+            
+}
+
 #
 # PRIVATE
 #
 
-sub _lastPart {    #function!
-    my $plugin = shift;                 #a class name
-    my @parts = split('::', $plugin);
-    return $parts[-1];
-}
-
 __PACKAGE__->meta->make_immutable;
 
 1;
+
 __END__
 =pod
 
@@ -79,21 +131,16 @@ Plugin::Tiny - A tiny plugin system for perl
 
 =head1 VERSION
 
-version 0.002
+version 0.003
 
 =head1 SYNOPSIS
 
-  #from core or bundling plugin
-  use Moose; #optional, t/tiny.t avoids using moose in core
+  #in your core
   use Plugin::Tiny; 
-  has 'plugins'=>(
-    is=>'ro',
-    isa=>'Plugin::Tiny', 
-    default=>sub{Plugin::Tiny->new()}
-  );
+  my $ps=Plugin::Tiny->new(); #plugin system
   
   #load plugin_class (and perhaps phase) from your configuration
-  $self->plugins->register(
+  $ps->register(
     phase=>$phase,         #optional; defaults to last part of plugin class
     plugin=>$plugin_class, #required
     role=>$role,           #optional
@@ -102,7 +149,7 @@ version 0.002
   );
 
   #execute your plugin's methods 
-  my $plugin=$self->get_plugin ($phase); 
+  my $plugin=$ps->get_plugin ($phase); 
   $plugin->do_something(@args);  
 
 =head1 DESCRIPTION
@@ -115,9 +162,16 @@ each phase can have only one plugin.
 
 You can still create bundles of plugins if you hand the plugin system down to 
 the (bundeling) plugin. That way, you can load multiple plugins for one 
-phase (althoughyou still need distinct phase labels for each plugin).
+phase (you still need distinct phase labels for each plugin).
 
   #in your core
+  use Moose; #optional
+  has 'plugins'=>(
+    is=>'ro',
+    isa=>'Plugin::Tiny', 
+    default=>sub{}
+  );
+
   $self->plugins->register(
     phase=>'Scan', 
     plugin=>'Plugin::ScanBundle', 
@@ -139,6 +193,10 @@ an interface etc.
 
 =head1 ATTRIBUTES
 
+=head2 debug
+
+expects a boolean. Prints additional info to STDOUT.
+
 =head2 prefix
 
 Optional init argument. You can have the prefix added to all plugin classes you
@@ -161,14 +219,20 @@ overwritten in register.
 
 =head1 METHODS
 
-=head2 $plugin_system->register(phase=>$phase, plugin=>$plugin_class);  
+=head2 $ps->register(phase=>$phase, plugin=>$plugin_class);  
+
+Registers a plugin, e.g. uses it and makes a new plugin object. Needs a
+plugin. If you don't specify a phase it, it uses a default phase from the 
+plugin class name. See method C<default_phae> for details.
 
 Optionally, you can also specify a role which your plugin will have to be able 
-to apply. Remaining key value pairs are passed down to the plugin constructor: 
+to apply. Specify role=>undef to unset global roles.
+
+Remaining key value pairs are passed down to the plugin constructor: 
 
   $plugin_system->register (
-    phase=>$phase,           #optional. Defaults to last part of plugin_class 
     plugin=>$plugin_class,   #required
+    phase=>$phase,           #optional
     role=>$role,             #optional
     plugins=>$plugin_system, #optional
     args=>$more_args,        #optional
@@ -179,9 +243,30 @@ named arguments.
 
 Returns the newly created plugin object on success. Confesses on error.
 
-=head2 my $plugin=$self->get_plugin ($phase);
+=head2 $plugin=$ps->get_plugin ($phase);
 
-Returns the plugin object associated with the phase.
+Returns the plugin object associated with the phase. Returns undef if no plugin
+is registered for this phase.
+
+=head2 $ps->defaultPhase ($plugin_class);
+
+Makes a default phase from a class name. If prefix is defined it use tail minus 
+'::'. Otherwise just last element of the class name.
+
+For My::Plugin::Long::Example and prefix='My::Plugin::' this results in 
+'Long::Example' and without prefix it would be 'Example'.
+
+Returns scalar;
+
+=head2 $class=$ps->get_class ($plugin); 
+
+returns the plugin's class. A bit like C<ref $plugin>. Not sure what it returns
+on error. Todo!
+
+=head2 $phase=$ps->get_phase ($plugin); 
+
+returns the plugin's phase. Returns undef on failure. Normally, you should not
+need this.
 
 =head1 AUTHOR
 
